{%- if pagename != "search" and builder != "singlehtml" %}
<div id="searchbox" style="display: none" role="search">
    <div class="searchformwrapper" style="margin-bottom:10%;">
        <form class="search" action="{{ pathto('search') }}" method="get">
            <input type="text" name="q" aria-labelledby="searchlabel" placeholder="Search" />
            <input type="submit" value="{{ _('Go') }}" />
        </form>
    </div>
</div>
<script type="text/javascript">$('#searchbox').show(0);</script>
{%- endif %}