# Generated by Django 4.0.3 on 2022-07-11 11:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0005_sequence_create_timestamp_sequence_update_timestamp"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="sequence",
            options={"ordering": ("-update_timestamp",)},
        ),
    ]
