# Generated by Django 4.0.3 on 2022-06-28 14:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0002_alter_contentitem_download_timestamp_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="contentitem",
            name="title",
            field=models.CharField(help_text="Title of content.", max_length=200),
        ),
        migrations.AlterField(
            model_name="obcontentitem",
            name="disqus_id",
            field=models.CharField(
                blank=True,
                help_text="Post Disqus API string identifier.",
                max_length=400,
            ),
        ),
        migrations.AlterField(
            model_name="obcontentitem",
            name="item_id",
            field=models.CharField(
                help_text='Post string identifier. E.g. "2006/11/introduction"',
                max_length=300,
                unique=True,
                verbose_name="string ID",
            ),
        ),
    ]
