# Generated by Django 4.0.3 on 2022-06-28 13:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("obapi", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="contentitem",
            name="download_timestamp",
            field=models.DateTimeField(
                editable=False,
                help_text="When the item was last downloaded.",
                null=True,
                verbose_name="download date",
            ),
        ),
        migrations.AlterField(
            model_name="contentitem",
            name="update_timestamp",
            field=models.DateTimeField(
                auto_now=True,
                help_text="When the item was last updated"
                " - either by hand or automatically.",
                verbose_name="update date",
            ),
        ),
        migrations.AlterField(
            model_name="externallink",
            name="url",
            field=models.URLField(help_text="Link URL.", max_length=2048),
        ),
    ]
